package com.nemrodsoftware.maven;

/*
 * Copyright 2001-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import com.nemrodsoftware.INemrodClient;
import com.nemrodsoftware.NemrodClient;
import com.nemrodsoftware.exception.OperationException;
import com.nemrodsoftware.model.Module;

/**
 * Goal which import resource file from a Nemrod instance
 *
 * @goal sync
 * 
 * @phase process-sources
 */
public class SyncMojo
    extends AbstractMojo
{
    /**
     * Source folder
     * @parameter expression="${sync.srcfolder}" default-value="${basedir}/src/main/resources"
     */
    private File srcfolder;   

    /**
     * Login.
     * @parameter expression="${sync.apikey}" 
     * @required
     */
    private String apikey;

    /**
     * Url.
     * @parameter expression="${sync.url}"  default-value="http://live.localizeyourapps.com"
     * @required
     */
    private String url;  
    
    /**
     * Project name.
     * @parameter expression="${sync.project}"
     * @required
     */
    private String projectName;        

    /**
     * sync format
     *
     * @parameter expression="${sync.format}" default-value="properties"
     * @required
     */
    private String format = "properties";         

    @Override
    public void execute()
        throws MojoExecutionException
    {
        try
        {
            INemrodClient client = new NemrodClient(url, apikey);
            
            getLog().info("Action : sync ");
            getLog().info("url="+url);
            getLog().info("project="+projectName);
            getLog().info("format="+format);
            getLog().info("workfolder="+srcfolder.getAbsolutePath());
            
            if (StringUtils.isEmpty(projectName))
                throw new IllegalArgumentException("Project name is not set. Please set sync.project property");
            
            List<Module> modules = retrieveModules(client);
            
            for (Module module : modules)
            {
                File masterFile = new File(srcfolder,module.getName()+".properties");                
                
                getLog().info("");
                getLog().info("Synchronize default bundle ["+module.getName()+"] for project ["+projectName+"] and path ["+module.getFullpath()+"]...");
                
                execute(client,masterFile,module);
            }            
        }
        catch (Exception e)
        {
            throw new MojoExecutionException("Unable to import resource bundles",e);
        }
    }
    
    public void execute(INemrodClient client, File masterFile, Module module) throws IOException, OperationException
    {
        if (masterFile == null || !masterFile.exists())
            throw new IllegalArgumentException("Master file is invalid. Please set a valid path to the master file sync.workfolder and sync.bundles property");
        
        client.sync(projectName, format, module, masterFile);
        
        getLog().info("Sync master file="+masterFile.getAbsolutePath()+" - OK");
    }
    
    private List<Module> retrieveModules(INemrodClient client) {
        return client.listModule(projectName);
    }
}
